ResultLayout := RECORD
	SET OF VARSTRING Rows;
END;

DATASET(ResultLayout) ExecuteSQL(VARSTRING DSN, VARSTRING Stmt) := BEGINC++
	#include <hpcc-odbc.h>
	#option library odbc
	#option action
	#define SQLMAXFIELDLEN 2056 // Length of the longest possible field
	
	#body

	__lenResult = 0;

	// Instantiate the class. This will also connect to the database.
	ODBC_Class instance(dsn);

	// Optionally, print out some driver information. This gets output to std::cout.
	//instance.getDriverInfo();

	// Execute the SQL statement. Note that since we cannot maintain state, we use SQLExecDirect 
	SQLRETURN rc = SQL_SUCCESS;
	rc = SQLExecDirect(instance.stmtHandle,(SQLCHAR *)
						stmt,SQL_NTS);
	if (!SQL_SUCCEEDED(rc)) {
		extract_error("SQLExecDirect",instance.stmtHandle,SQL_HANDLE_STMT);
		return;
	}
	SQLSMALLINT columns;
	SQLNumResultCols(instance.stmtHandle,&columns);

	// Container to hold the row -- this must be an auto object for the memory to free automatically
	vector< vector<string> > rows;
	vector<size_t> setSizes;
	
	char buf[SQLMAXFIELDLEN];
	size_t size = 0;
	size_t numRows = 0;
	while(SQL_SUCCEEDED(rc = SQLFetch(instance.stmtHandle))) {
		// Container to hold the columns in the row
		numRows++;
		vector<string> row;
		row.reserve(columns);
		size += 1; // +1 for the initial null marker
		size += 4; // +4 for prefix length of the set
		size_t setSize = 0;
		for (int i=1; i<=columns; i++) {
			SQLLEN indicator;
			rc = SQLGetData(instance.stmtHandle,i,SQL_C_CHAR,buf,sizeof(buf),&indicator);
			if (SQL_SUCCEEDED(rc)) {
				if (indicator == SQL_NULL_DATA) {
					strcpy(buf,"NULL");
					indicator = 4;
				}
				row.push_back(string(buf));
				size_t increment  = (indicator +1); // +1 for the null terminator in VARSTRING or +4 for length prefix in STRING
				size += increment;
				setSize += increment;
			}
		}
		rows.push_back(row);
		setSizes.push_back(setSize);
	}
	
	// Allocate permanent memory for the dataset
	__result = rtlMalloc(size);
	__lenResult = size;

	// Now serialize the rows.
	byte *ptr = (byte *)__result;
	for (int i=0; i<numRows; i++) {
		bzero(ptr,1); ptr += 1; // Set null marker
		memcpy(ptr,(void *)&setSizes[i],4); ptr += 4; // Set the prefix length for the set
		vector <string> *row = &rows[i];
		for (int j=0; j<row->size(); j++) {
			int len = ((*row)[j]).length();
			memcpy(ptr,((*row)[j]).c_str(),len+1); // Copy contents and return pointer to the terminating null byte of the new string.
			ptr += len + 1; // Move the pointer one past the null byte
		}
	}

	// The vector objects will free themselves.
	// The class destructor will call itself and close all connections as our instance is an auto object.
	return;
ENDC++;

ExecuteSQL('MySQL-Test','SELECT * FROM hotlist LIMIT 5;');

